/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                                SDK DEMO                                   */
/*                     Simple edit box for frequencies                       */
/*                        (c) Copyright RADIXON 2020                         */
/*****************************************************************************/

#include "u_frequencyedit.h"
#include <wx/tokenzr.h>

TFrequencyEdit::TFrequencyEdit(wxWindow *Parent,wxWindowID Id,int64_t Value,int64_t MinValue,int64_t MaxValue,
    const wxPoint &Position,const wxSize &Size):TNumericEdit(Parent,Id,Value,MinValue,MaxValue,Position,Size)
{
    m_Units=FrequencyEditUnits_MHz;
    m_AutoUnits=false;

    UpdateText();
}

TFrequencyEdit::~TFrequencyEdit(void)
{
}


wxString TFrequencyEdit::Format(int64_t Value)
{
 wxString s;

    switch(m_Units)
    {
        case FrequencyEditUnits_Hz:
            s.sprintf(wxT("%Ld Hz"),Value);
            break;

        case FrequencyEditUnits_kHz:
            s.sprintf(wxT("%.3f kHz"),Value*(1.0/1000.0));
            break;

        case FrequencyEditUnits_MHz:
            s.sprintf(wxT("%.6f MHz"),Value*(1.0/1000000.0));
            break;

        case FrequencyEditUnits_GHz:
            s.sprintf(wxT("%.9f GHz"),Value*(1.0/1000000000.0));
            break;
    }

    return s;
}


bool TFrequencyEdit::Parse(const wxString &Str,int64_t *Value)
{
 static struct
 {
     const wxChar           *Abbr;
     const wxChar           *Text;
     int                    Mul;
     TFrequencyEditUnits    Code;
 } Units[]=
 {
     {wxT("H"),wxT("Hz"),1,FrequencyEditUnits_Hz},
     {wxT("k"),wxT("kHz"),1000,FrequencyEditUnits_kHz},
     {wxT("M"),wxT("MHz"),1000000,FrequencyEditUnits_MHz},
     {wxT("G"),wxT("GHz"),1000000000,FrequencyEditUnits_GHz}
 };

 char *End;
 double fFrequency;
 wxString StrUnits;
 size_t i;
 char buf[128];


    strcpy(buf,Str.mb_str(wxConvLocal));

    fFrequency=strtod(buf,&End);

    StrUnits=wxString(End,wxConvLocal).Trim(true).Trim(false);

    if(StrUnits.length()==0)
    {
        i=0;
    }
    else
    {
        for(i=0;i<sizeof(Units)/sizeof(Units[0]);i++)
        {
            if(StrUnits.CmpNoCase(Units[i].Abbr)==0 || StrUnits.CmpNoCase(Units[i].Text)==0)
            {
                break;
            }
        }

        if(i>=sizeof(Units)/sizeof(Units[0]))
        {
            return false;
        }
    }


    *Value=(int64_t)(ceil(fFrequency*Units[i].Mul));

    if(m_AutoUnits)
    {
        m_Units=Units[i].Code;
    }

    return true;
}
