/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                          Coherent mode SDK DEMO                           */
/*           Device set selection and "No device set found" dialogs          */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#include "u_devicesetselector.h"
#include "u_defs.h"
#include "u_global.h"

extern G65DDC_CREATE_INSTANCE CreateInstance;

TNoDeviceSetFoundWindow::TNoDeviceSetFoundWindow(void):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(1000,1000))
{
 wxStaticText *st;
 wxSizer *Sizer,*ButtonSizer;
 size_t i;
 static const wxChar *Msg[]={wxT("No coherent G65DDC device set has been found."),
                             wxT(""),
                             wxT("This may mean that either no receiver is connected,"),
                             wxT("or it is not powered, or its driver was not"),
                             wxT("loaded in time before this application was executed.")};

    SetFont(*Global.m_Font);

    Sizer=new wxBoxSizer(wxVERTICAL);

    Sizer->AddSpacer(20);

    for(i=0;i<sizeof(Msg)/sizeof(Msg[0]);i++)
    {
        st=new wxStaticText(this,wxID_ANY,Msg[i],wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE_HORIZONTAL);
        Sizer->Add(st,0,wxALIGN_TOP|wxALIGN_CENTER_HORIZONTAL|wxLEFT|wxRIGHT,0);
    }

    ButtonSizer=new wxBoxSizer(wxHORIZONTAL);


    m_BTryAgain=new wxButton(this,wxID_RETRY,wxT("Try again"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    ButtonSizer->Add(m_BTryAgain,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BDemo=new wxButton(this,wxID_DEFAULT,wxT("Run in demo mode"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    ButtonSizer->Add(m_BDemo,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    ButtonSizer->Add(m_BCancel,0,wxALIGN_CENTER);

    Sizer->Add(ButtonSizer,0,wxALIGN_TOP|wxALL,20);

    SetSizerAndFit(Sizer);

    CenterOnScreen();
}

TNoDeviceSetFoundWindow::~TNoDeviceSetFoundWindow(void)
{
}

void TNoDeviceSetFoundWindow::OnTryAgain(wxCommandEvent &Event)
{
    EndModal(wxID_RETRY);
}

void TNoDeviceSetFoundWindow::OnDemo(wxCommandEvent &Event)
{
    EndModal(wxID_DEFAULT);
}

BEGIN_EVENT_TABLE(TNoDeviceSetFoundWindow, wxDialog)
    EVT_BUTTON(wxID_RETRY,TNoDeviceSetFoundWindow::OnTryAgain)
    EVT_BUTTON(wxID_DEFAULT,TNoDeviceSetFoundWindow::OnDemo)
END_EVENT_TABLE()

TDeviceSetSelector::TDeviceSetSelector(ICohG65DDCDeviceSetEnumerator *Enumerator):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 uint32_t i,DeviceCount,j;
 G65DDC_DEVICE_INFO *DeviceInfos;
 wxString *list,s;
 wxSizer *Sizer,*MainSizer;

    SetFont(*Global.m_Font);

    MainSizer=new wxBoxSizer(wxHORIZONTAL);

    list=new wxString[Enumerator->GetDeviceSetCount()];

    for(i=0;i<Enumerator->GetDeviceSetCount();i++)
    {
        DeviceCount=0;
        Enumerator->GetDeviceSetInfo(i,NULL,&DeviceCount);

        DeviceInfos=new G65DDC_DEVICE_INFO[DeviceCount];

        Enumerator->GetDeviceSetInfo(i,DeviceInfos,&DeviceCount);

        s=wxString(DeviceInfos[0].SerialNumber,wxConvLocal);

        for(j=1;j<DeviceCount;j++)
        {
            s=s+wxT("/")+wxString(DeviceInfos[j].SerialNumber,wxConvLocal);
        }

        list[i]=s;

        delete[] DeviceInfos;
    }

    m_RadioBox=new wxRadioBox(this,wxID_ANY,wxT("Select device set to open"),wxDefaultPosition,wxDefaultSize,Enumerator->GetDeviceSetCount(),list,0,wxVERTICAL);

    MainSizer->Add(m_RadioBox,0,wxEXPAND|wxALL,20);

    delete[] list;

    Sizer=new wxBoxSizer(wxVERTICAL);

    MainSizer->Add(Sizer,0,wxTOP|wxRIGHT,20);

    m_BOK=new wxButton(this,wxID_OK,wxT("OK"));
    Sizer->Add(m_BOK);
    Sizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"));
    Sizer->Add(m_BCancel);


    m_DeviceSetIndex=m_RadioBox->GetSelection();

    SetSizerAndFit(MainSizer);

    CenterOnScreen();
}

TDeviceSetSelector::~TDeviceSetSelector(void)
{
}

void TDeviceSetSelector::OnOk(wxCommandEvent &Event)
{
    m_DeviceSetIndex=m_RadioBox->GetSelection();
    EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TDeviceSetSelector, wxDialog)
    EVT_BUTTON(wxID_OK,TDeviceSetSelector::OnOk)
END_EVENT_TABLE()


ICohG65DDCDeviceSet* TDeviceSetSelector::SelectDeviceSet(void)
{
 ICohG65DDCDeviceSetEnumerator *Enumerator;
 TNoDeviceSetFoundWindow *NoDeviceSet;
 ICohG65DDCDeviceSet *DeviceSet;
 TDeviceSetSelector *Selector;
 uint32_t Count,DeviceCount;
 int r;
 G65DDC_DEVICE_INFO *DeviceInfos;

    if(!CreateInstance(G65DDC_CLASS_ID_COH_DEVICE_SET_ENUMERATOR,(void**)&Enumerator))
    {
        wxMessageBox(wxT("Failed to create device set enumerator object!"),APP_TITLE,wxICON_ERROR);
        return NULL;
    }

    if(!CreateInstance(G65DDC_CLASS_ID_COH_DEVICE_SET,(void**)&DeviceSet))
    {
        wxMessageBox(wxT("Failed to create device set object!"),APP_TITLE,wxICON_ERROR);
        Enumerator->Free();
        return NULL;
    }

    for(;;)
    {
        Count=Enumerator->GetDeviceSetCount();

        if(Count==0) //no device set has been found
        {
            NoDeviceSet=new TNoDeviceSetFoundWindow;

            r=NoDeviceSet->ShowModal();

            delete NoDeviceSet;

            if(r==wxID_RETRY)
            {
                Enumerator->Enumerate();
            }
            else if(r==wxID_DEFAULT)
            {
                if(DeviceSet->Open(COH_G65DDC_OPEN_DEMO,3))
                {
                    Enumerator->Free();
                    return DeviceSet;
                }

                wxMessageBox(wxT("Failed to open demo device set!"),APP_TITLE,wxICON_ERROR);
                break;
            }
            else
            {
                break;
            }
        }
        else if(Count==1) //single device set has been found
        {
            DeviceCount=0;
            Enumerator->GetDeviceSetInfo(0,NULL,&DeviceCount);

            DeviceInfos=new G65DDC_DEVICE_INFO[DeviceCount];

            Enumerator->GetDeviceSetInfo(0,DeviceInfos,&DeviceCount);

            if(DeviceSet->Open(DeviceInfos,DeviceCount))
            {
                //device set has been open
                delete[] DeviceInfos;
                Enumerator->Free();
                return DeviceSet;
            }

            delete[] DeviceInfos;

            wxMessageBox(wxT("Failed to open device set!"),APP_TITLE,wxICON_ERROR);
            break;
        }
        else
        {
            //more than one device set has been detected
            Selector=new TDeviceSetSelector(Enumerator);

            if(Selector->ShowModal()!=wxID_OK)
            {
                delete Selector;
                break;
            }

            r=Selector->GetSelectedDeviceSetIndex();

            delete Selector;

            DeviceCount=0;
            Enumerator->GetDeviceSetInfo(r,NULL,&DeviceCount);
            DeviceInfos=new G65DDC_DEVICE_INFO[DeviceCount];


            Enumerator->GetDeviceSetInfo(r,DeviceInfos,&DeviceCount);

            if(DeviceSet->Open(DeviceInfos,DeviceCount))
            {
                delete[] DeviceInfos;
                Enumerator->Free();
                return DeviceSet;
            }

            delete[] DeviceInfos;
            wxMessageBox(wxT("Failed to open device set!"),APP_TITLE,wxICON_ERROR);
            break;
        }
    }

    Enumerator->Free();
    DeviceSet->Free();

    return NULL;
}
