/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                          Coherent mode SDK DEMO                           */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_global.h"


enum
{
    ID_DDC1_FREQUENCY=wxID_HIGHEST,
    ID_DDC1_BW,
    ID_POWER,
    ID_1PPS,
    ID_INIT_POWER_ON_TIMER,
    ID_TIMESTAMP_TIMER,
    ID_DEVICE_STATE_TIMER
};

BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick)
    EVT_TOGGLEBUTTON(ID_1PPS,TMainWindow::OnButtonClick)
    EVT_COMBOBOX(ID_DDC1_BW,TMainWindow::OnComboBoxChanged)
    EVT_NUMERICEDIT(ID_DDC1_FREQUENCY,TMainWindow::OnNumericEditChanged)
    EVT_TIMER(ID_INIT_POWER_ON_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_TIMESTAMP_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_DEVICE_STATE_TIMER,TMainWindow::OnTimer)
END_EVENT_TABLE()


TMainWindow::TMainWindow(ICohG65DDCDeviceSet *DeviceSet):wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,900)),
m_DeviceSet(DeviceSet),
m_InitPowerOnTimer(this,ID_INIT_POWER_ON_TIMER),
m_TimestampTimer(this,ID_TIMESTAMP_TIMER),
m_DeviceStateTimer(this,ID_DEVICE_STATE_TIMER)
{
 uint32_t i,j,k,Count;
 wxString s;
 wxSizer *MainSizer,*MidSizer,*Sizer,*DeviceSizer,*PhaseSizer;
 int32_t Frequency;
 G65DDC_DDC_INFO DDCInfo;
 wxStaticText *Text;
 wxSize Size;

    m_PrevADCCounter=0;
    m_PrevDDC1Counter=0;

    m_DeviceSet->GetDeviceCount(&m_DeviceCount);

    m_DeviceInfos=new G65DDC_DEVICE_INFO[m_DeviceCount];
    m_DeviceSet->GetDeviceInfos(m_DeviceInfos,&m_DeviceCount);

    m_DevicePairCount=m_DeviceCount*(m_DeviceCount-1)/2;

    s=APP_TITLE;
    s=s+wxT(" - ")+wxString(m_DeviceInfos[0].SerialNumber,wxConvLocal);

    for(i=1;i<m_DeviceCount;i++)
    {
        s=s+wxT(" / ")+wxString(m_DeviceInfos[i].SerialNumber,wxConvLocal);
    }

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    SetTitle(s);

    MainSizer=new wxBoxSizer(wxVERTICAL);
    SetSizer(MainSizer);

    Sizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(Sizer,0,wxEXPAND|wxALIGN_TOP|wxALL,4);

    m_LDDC1Frequency=new wxStaticText(this,wxID_ANY,wxT("DDC1 frequency"));
    Sizer->Add(m_LDDC1Frequency,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_DeviceSet->GetDDC1Frequency(&Frequency);
    m_EDDC1Frequency=new TFrequencyEdit(this,ID_DDC1_FREQUENCY,Frequency,m_DeviceInfos[0].Ranges[G65DDC_RANGE_1].MinFrequency,m_DeviceInfos[0].Ranges[G65DDC_RANGE_2].MaxFrequency,wxDefaultPosition,Size);

    Size=m_EDDC1Frequency->GetTextExtent(wxT("999.999999 MHz"));
    Size=m_EDDC1Frequency->GetSizeFromTextSize(Size.x+20);

    m_EDDC1Frequency->SetInitialSize(Size);

    Sizer->Add(m_EDDC1Frequency,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL );

    Sizer->AddSpacer(8);

    m_LDDC1BW=new wxStaticText(this,wxID_ANY,wxT("DDC1 BW"));
    Sizer->Add(m_LDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);


    m_CBDDC1BW=new wxComboBox(this,ID_DDC1_BW,wxEmptyString,wxDefaultPosition,wxDefaultSize,0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);

    Size=m_CBDDC1BW->GetTextExtent(wxT("99.9 MHz"));
    Size=m_CBDDC1BW->GetSizeFromTextSize(Size.x+20);
    m_CBDDC1BW->SetInitialSize(Size);

    Sizer->Add(m_CBDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DeviceSet->GetDDC1Count(&Count);

    for(i=0;i<Count;i++)
    {
        m_DeviceSet->GetDDCInfo(i,&DDCInfo);

        m_CBDDC1BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));
    }


    MidSizer=new wxBoxSizer(wxHORIZONTAL);
    MainSizer->Add(MidSizer,1,wxEXPAND|wxLEFT|wxRIGHT,4);

    DeviceSizer=new wxBoxSizer(wxVERTICAL);
    MidSizer->Add(DeviceSizer,1,wxEXPAND|wxALIGN_LEFT);

    PhaseSizer=new wxBoxSizer(wxVERTICAL);
    MidSizer->Add(PhaseSizer,1,wxEXPAND|wxLEFT,8);

    m_DevicePanels=new TDevicePanel*[m_DeviceCount];

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]=new TDevicePanel(this,m_DeviceSet,i);
        DeviceSizer->Add(m_DevicePanels[i],1,wxEXPAND);
    }

    m_Phases=new TPhase*[m_DevicePairCount];

    for(i=0,j=0;j<m_DeviceCount;j++)
    {
        for(k=j+1;k<m_DeviceCount;k++)
        {
            s.sprintf(wxT("Phase %s - %s"),wxString(m_DeviceInfos[j].SerialNumber,wxConvLocal).c_str(),
                wxString(m_DeviceInfos[k].SerialNumber,wxConvLocal).c_str());

            Text=new wxStaticText(this,wxID_ANY,s);
            PhaseSizer->Add(Text,0,wxALIGN_LEFT|wxTOP|wxBOTTOM,4);

            m_Phases[i]=new TPhase(this,wxID_ANY);
            PhaseSizer->Add(m_Phases[i],1,wxEXPAND);

            i++;
        }
    }

    Sizer=new wxBoxSizer(wxHORIZONTAL);

    MainSizer->Add(Sizer,0,wxEXPAND|wxALL,4);


    m_B1PPS=new wxToggleButton(this,ID_1PPS,wxT("1PPS Enabled"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    Sizer->Add(m_B1PPS,0,wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL);

    Sizer->AddSpacer(16);

    m_LDDC1Counter=new wxStaticText(this,wxID_ANY,wxT("DDC1 Counter:"));
    Sizer->Add(m_LDDC1Counter,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LDDC1CounterValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(100,m_LDDC1Counter->GetSize().GetHeight()),wxST_NO_AUTORESIZE);
    Sizer->Add(m_LDDC1CounterValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LADCCounter=new wxStaticText(this,wxID_ANY,wxT("ADC Counter:"));
    Sizer->Add(m_LADCCounter,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LADCCounterValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(120,m_LADCCounter->GetSize().GetHeight()),wxST_NO_AUTORESIZE);
    Sizer->Add(m_LADCCounterValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LDDC1SampleRate=new wxStaticText(this,wxID_ANY,wxT("DDC1 Samplerate:"));
    Sizer->Add(m_LDDC1SampleRate,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LDDC1SampleRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(120,m_LDDC1SampleRate->GetSize().GetHeight()),wxST_NO_AUTORESIZE);
    Sizer->Add(m_LDDC1SampleRateValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(8);

    m_LADCSampleRate=new wxStaticText(this,wxID_ANY,wxT("ADC Samplerate:"));
    Sizer->Add(m_LADCSampleRate,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddSpacer(4);

    m_LADCSampleRateValue=new wxStaticText(this,wxID_ANY,wxT("N/A"),wxDefaultPosition,wxSize(120,m_LADCSampleRate->GetSize().GetHeight()),wxST_NO_AUTORESIZE);
    Sizer->Add(m_LADCSampleRateValue,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    Sizer->AddStretchSpacer();

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
    Sizer->Add(m_BPower,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL);

    m_DeviceSet->GetDDC1(&i,&m_DDC1Info);
    m_CBDDC1BW->SetSelection(i);
    m_DeviceSet->SetCallback(this);

    SetClientSize(1200,850);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    Fit();
    Centre();

    m_InitPowerOnTimer.Start(200,true);
    m_DeviceStateTimer.Start(500);
}

TMainWindow::~TMainWindow(void)
{
    m_DeviceSet->Free();

    delete[] m_DeviceInfos;
    delete[] m_DevicePanels;
}


void TMainWindow::CohG65DDC_DDC1StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceCount,const void **Buffers,uint32_t Count,uint32_t BitsPerSample)
{
 uint32_t i,j,k;
 float From,To;

    m_DDC1SampleCounter+=Count;

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->DDC1StreamCallback(Buffers[i],Count,BitsPerSample);
    }

    if(BitsPerSample==16)
    {
        for(i=0,j=0;j<m_DeviceCount;j++)
        {
            for(k=j+1;k<m_DeviceCount;k++)
            {
                GetPhase_int16((int16_t*)Buffers[j],(int16_t*)Buffers[k],Count,&From,&To);

                m_Phases[i]->Add(From,To);

                i++;
            }
        }
    }
    else
    {
        for(i=0,j=0;j<m_DeviceCount;j++)
        {
            for(k=j+1;k<m_DeviceCount;k++)
            {
                GetPhase_int32((int32_t*)Buffers[j],(int32_t*)Buffers[k],Count,&From,&To);

                m_Phases[i]->Add(From,To);

                i++;
            }
        }
    }
}

void TMainWindow::CohG65DDC_ADCSnapshotCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel)
{
    m_DevicePanels[DeviceIndex]->ADCSnapshotCallback(Buffer,Count,CenterFrequency,ADCLevel);
}

void TMainWindow::CohG65DDC_DDC2StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count)
{
}

void TMainWindow::CohG65DDC_DDC2PreprocessedStreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count,float Peak,float RMS)
{
}

void TMainWindow::CohG65DDC_AudioStreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,uint32_t Stage,const float *Buffer,uint32_t Count)
{
}

void TMainWindow::SetPower(bool Power)
{
 uint32_t i;
 uint32_t SamplesPerBuffer;
 wxBusyCursor *BusyCursor;

    if(Power)
    {
        m_PrevADCCounter=0;
        m_PrevDDC1Counter=0;
        BusyCursor=new wxBusyCursor;
    }

    m_BPower->SetValue(Power);
    m_BPower->Update();

    m_DeviceSet->SetPower(Power);

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->UpdateControls();
    }

    if(Power)
    {
        m_DDC1SampleCounter=0;

        if(m_DDC1Info.Bandwidth<4000000)
        {
            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+63)&(~63);
        }
        else
        {
            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+1023)&(~1023);
        }

        m_DeviceSet->StartDDC1(SamplesPerBuffer);

        for(i=0;i<m_DeviceCount;i++)
        {
            m_DeviceSet->StartADCSnapshots(i,UPDATE_INTERVAL,G65DDC_ADC_SAMPLES_PER_SNAPSHOT_64K);
        }

        m_TimestampTimer.Start(500);

        delete BusyCursor;
    }
    else
    {
        m_TimestampTimer.Stop();
    }

    m_B1PPS->Enable(!Power); //1PPS functionality can be enabled/disabled only when the device set if turned off

    m_LDDC1Frequency->Enable(Power);
    m_EDDC1Frequency->Enable(Power);
    m_LDDC1BW->Enable(Power);
    m_CBDDC1BW->Enable(Power);

    for(i=0;i<m_DevicePairCount;i++)
    {
        m_Phases[i]->Enable(Power);
        m_Phases[i]->Clear();
    }

    Update1PPSControls();
}

void TMainWindow::Update1PPSControls(void)
{
 int Power,OnePPS;
 bool Enabled;

    m_DeviceSet->GetPower(&Power);
    m_DeviceSet->Get1PPSEnabled(&OnePPS);

    Enabled=Power && OnePPS;

    m_LDDC1Counter->Enable(Enabled);
    m_LDDC1CounterValue->Enable(Enabled);
    m_LDDC1CounterValue->SetLabel(wxT("N/A"));

    m_LADCCounter->Enable(Enabled);
    m_LADCCounterValue->Enable(Enabled);
    m_LADCCounterValue->SetLabel(wxT("N/A"));

    m_LDDC1SampleRate->Enable(Enabled);
    m_LDDC1SampleRateValue->Enable(Enabled);
    m_LDDC1SampleRateValue->SetLabel(wxT("N/A"));

    m_LADCSampleRate->Enable(Enabled);
    m_LADCSampleRateValue->Enable(Enabled);
    m_LADCSampleRateValue->SetLabel(wxT("N/A"));
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_POWER:
            SetPower(m_BPower->GetValue());
            break;

        case ID_1PPS:
            m_DeviceSet->Set1PPSEnabled(m_B1PPS->GetValue());
            break;
    }
}

void TMainWindow::SetDDC1Frequency(uint32_t Frequency)
{
 uint32_t i;
 uint32_t MinFrequency,MaxFrequency;
 uint32_t CurrentRange,Range;
 int32_t Diff_1,Diff_2;

    MinFrequency=m_DeviceInfos[0].Ranges[G65DDC_RANGE_1].MinFrequency+m_DDC1Info.Bandwidth/2;
    MaxFrequency=m_DeviceInfos[0].Ranges[G65DDC_RANGE_2].MaxFrequency-m_DDC1Info.Bandwidth/2;

    if(Frequency<MinFrequency)
    {
        Frequency=MinFrequency;
    }
    else if(Frequency>MaxFrequency)
    {
        Frequency=MaxFrequency;
    }

    m_DeviceSet->GetRange(&CurrentRange);

    Diff_1=m_DeviceInfos[0].Ranges[G65DDC_RANGE_1].MaxFrequency-Frequency;
    Diff_2=Frequency-m_DeviceInfos[0].Ranges[G65DDC_RANGE_2].MinFrequency;

    if(Diff_1>0)
    {
        //Frequency is in range1
        Range=G65DDC_RANGE_1;
    }
    else if(Diff_2>0)
    {
        //Frequency is in range2
        Range=G65DDC_RANGE_2;
    }
    else
    {
        //Frequency is between ranges, select the closest range
        if(abs(Diff_1)<abs(Diff_2))
        {
            Range=G65DDC_RANGE_1;
            Frequency=m_DeviceInfos[0].Ranges[G65DDC_RANGE_1].MaxFrequency-m_DDC1Info.Bandwidth/2;
        }
        else
        {
            Range=G65DDC_RANGE_2;
            Frequency=m_DeviceInfos[0].Ranges[G65DDC_RANGE_2].MinFrequency+m_DDC1Info.Bandwidth/2;
        }
    }

    if(Range!=CurrentRange)
    {
        m_DeviceSet->SetRange(Range);

        for(i=0;i<m_DeviceCount;i++)
        {
            m_DevicePanels[i]->UpdateRange();
        }
    }

    m_DeviceSet->SetDDC1Frequency(Frequency);

    m_EDDC1Frequency->Set(Frequency);

    for(i=0;i<m_DeviceCount;i++)
    {
        m_DevicePanels[i]->UpdateDDC1Frequency();
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
 uint32_t SamplesPerBuffer,i;

    switch(Event.GetId())
    {
        case ID_DDC1_BW:
            m_DeviceSet->StopDDC1();

            m_DeviceSet->SetDDC1(m_CBDDC1BW->GetSelection());
            m_DeviceSet->GetDDC1(NULL,&m_DDC1Info);

            SamplesPerBuffer=(m_DDC1Info.SampleRate*UPDATE_INTERVAL/1000+63)&(~63);

            for(i=0;i<m_DeviceCount;i++)
            {
                m_DevicePanels[i]->UpdateDDC1();
            }

            m_PrevADCCounter=0;
            m_PrevDDC1Counter=0;

            m_DeviceSet->StartDDC1(SamplesPerBuffer);
            break;
    }
}

void TMainWindow::OnNumericEditChanged(wxNotifyEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_DDC1_FREQUENCY:
            SetDDC1Frequency((uint32_t)m_EDDC1Frequency->Get());
            break;
    }
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 double DDC1Counter;
 uint64_t ADCCounter;
 wxString s;
 G65DDC_DEVICE_STATE DeviceState;

    switch(Event.GetId())
    {
        case ID_INIT_POWER_ON_TIMER:
            SetPower(true);
            break;

        case ID_DEVICE_STATE_TIMER:
            for(uint32_t i=0;i<m_DeviceCount;i++)
            {
                m_DevicePanels[i]->UpdateDeviceState();
            }
            break;

        case ID_TIMESTAMP_TIMER:
            if(m_DeviceSet->GetDeviceState(0,&DeviceState))
            {
                if((DeviceState.Flags & G65DDC_DEVICE_STATE_NO_1PPS)==0 && m_DeviceSet->Get1PPSDDC1Counters(&DDC1Counter,&ADCCounter))
                {
                    if(ADCCounter!=m_PrevADCCounter)
                    {
                        s.sprintf(wxT("%.1f"),DDC1Counter);
                        m_LDDC1CounterValue->SetLabel(s);

                        s.sprintf(wxT("%llu"),(long long)ADCCounter);
                        m_LADCCounterValue->SetLabel(s);

                        if(m_PrevADCCounter!=0)
                        {
                            s.sprintf(wxT("%.6f MHz"),(double)(ADCCounter-m_PrevADCCounter)*(1.0/1000000.0));
                            m_LADCSampleRateValue->SetLabel(s);
                        }

                        if(m_PrevDDC1Counter!=0)
                        {
                            s.sprintf(wxT("%.6f MHz"),(DDC1Counter-m_PrevDDC1Counter)*(1.0/1000000.0));
                            m_LDDC1SampleRateValue->SetLabel(s);
                        }

                        m_PrevADCCounter=ADCCounter;
                        m_PrevDDC1Counter=DDC1Counter;
                    }
                }
            }
            break;
    }
}

void TMainWindow::GetPhase_int16(int16_t *Buffer1,int16_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To)
{
 uint32_t i;
 float I1,Q1,I2,Q2;
 float p;

    if(SamplesPerBuffer>PHASE_MAX_SAMPLES)
    {
        SamplesPerBuffer=PHASE_MAX_SAMPLES;
    }

    I1=Buffer1[0]*(1.0/32768.0);
    Q1=Buffer1[1]*(1.0/32768.0);
    I2=Buffer2[0]*(1.0/32768.0);
    Q2=Buffer2[1]*(1.0/32768.0);

    p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

    *From=*To=p;

    for(i=1;i<SamplesPerBuffer;i++)
    {
        I1=Buffer1[i*2]*(1.0/32768.0);
        Q1=Buffer1[i*2+1]*(1.0/32768.0);
        I2=Buffer2[i*2]*(1.0/32768.0);
        Q2=Buffer2[i*2+1]*(1.0/32768.0);

        p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

        if(p<*From)
        {
            *From=p;
        }
        else if(p>*To)
        {
            *To=p;
        }
    }
}

void TMainWindow::GetPhase_int32(int32_t *Buffer1,int32_t *Buffer2,uint32_t SamplesPerBuffer,float *From,float *To)
{
 uint32_t i;
 float I1,Q1,I2,Q2;
 float p;

    if(SamplesPerBuffer>PHASE_MAX_SAMPLES)
    {
        SamplesPerBuffer=PHASE_MAX_SAMPLES;
    }

    I1=Buffer1[0]*(1.0/2147483648);
    Q1=Buffer1[1]*(1.0/2147483648);
    I2=Buffer2[0]*(1.0/2147483648);
    Q2=Buffer2[1]*(1.0/2147483648);

    p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

    *From=*To=p;

    for(i=1;i<SamplesPerBuffer;i++)
    {
        I1=Buffer1[i*2]*(1.0/2147483648);
        Q1=Buffer1[i*2+1]*(1.0/2147483648);
        I2=Buffer2[i*2]*(1.0/2147483648);
        Q2=Buffer2[i*2+1]*(1.0/2147483648);

        p=atan2((double)(I1*Q2-Q1*I2),(double)(I1*I2+Q1*Q2))*180/M_PI;

        if(p<*From)
        {
            *From=p;
        }
        else if(p>*To)
        {
            *To=p;
        }
    }
}
