/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                           Configuration window                            */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#include "u_config_window.h"
#include "u_mainwindow.h"

enum
{
    ID_LIST=wxID_HIGHEST,
    ID_DELAY,
    ID_CONNECT
};

BEGIN_EVENT_TABLE(config_window, wxMiniFrame)
    EVT_BUTTON(ID_CONNECT,config_window::on_click_connect)
END_EVENT_TABLE()

config_window::config_window(main_window *window):wxMiniFrame(window,wxID_ANY,wxT("Configuration"),wxDefaultPosition)
{
 wxSizer *main_sizer,*sizer;
 wxSize size;

    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    m_main_window=window;

    main_sizer=new wxBoxSizer(wxVERTICAL);

    m_lv=new wxListCtrl(this,ID_LIST,wxDefaultPosition,wxDefaultSize,wxLC_REPORT);
    main_sizer->Add(m_lv,1,wxEXPAND|wxLEFT|wxTOP|wxRIGHT|wxBOTTOM,8);

    m_lv->AppendColumn(wxT("Index"),wxLIST_FORMAT_RIGHT);
    m_lv->AppendColumn(wxT("S/N"),wxLIST_FORMAT_LEFT);
    m_lv->AppendColumn(wxT("Delay"),wxLIST_FORMAT_RIGHT);
    m_lv->AppendColumn(wxT("Host name"),wxLIST_FORMAT_LEFT,wxLIST_AUTOSIZE);
    m_lv->AppendColumn(wxT("Address"),wxLIST_FORMAT_LEFT);
    m_lv->AppendColumn(wxT("Port"),wxLIST_FORMAT_RIGHT);

    sizer=new wxBoxSizer(wxHORIZONTAL);

    main_sizer->Add(sizer,0,wxEXPAND|wxLEFT|wxBOTTOM|wxRIGHT|wxALIGN_TOP,8);

    sizer->AddStretchSpacer();

    m_bconnect=new wxButton(this,ID_CONNECT,wxT("Connect"));
    sizer->Add(m_bconnect,0,wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT);

    SetSizerAndFit(main_sizer);

    update_list();
}

config_window::~config_window(void)
{

}

void config_window::update_list(void)
{
 const std::vector<rx_client*> &clients=m_main_window->get_clients();
 wxString s;
 long index;

    m_lv->DeleteAllItems();

    for(auto client:clients)
    {
        s.sprintf(wxT("%u"),client->get_index());
        index=m_lv->InsertItem(m_lv->GetItemCount(),s);

        m_lv->SetItem(index,1,client->get_serial_number().c_str());
        s.sprintf(wxT("%d"),client->get_delay());
        m_lv->SetItem(index,2,s);
        m_lv->SetItem(index,3,client->get_host_name());
        m_lv->SetItem(index,4,client->get_address_str());

        s.sprintf(wxT("%u"),client->get_port());
        m_lv->SetItem(index,5,s);
    }
}

void config_window::update_serial_numbers(void)
{
 const std::vector<rx_client*> &clients=m_main_window->get_clients();

    for(size_t i=0;i<clients.size();i++)
    {
        if((int)i<m_lv->GetItemCount() && clients[i]->get_serial_number_changed())
        {
            m_lv->SetItem(i,1,clients[i]->get_serial_number().c_str());
            m_main_window->update_rx_button(i);
        }
    }
}

void config_window::on_click_connect(wxCommandEvent &event)
{
    m_main_window->connect_disconnect();
}

void config_window::connection_state_changed(void)
{
    m_bconnect->SetLabel(m_main_window->is_connected()?wxT("Disconnect"):wxT("Connect"));
}
