/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#include <arpa/inet.h>
#include <netdb.h>
#include "u_mainwindow.h"
#include "u_rx_client.h"

class app: public wxApp
{
public:
    app(void);
    ~app(void);

    bool OnInit(void);
private:
    bool read_config(std::vector<remote_host*> &remote_hosts);
};

app::app(void)
{
}

app::~app(void)
{
}


bool app::read_config(std::vector<remote_host*> &remote_hosts)
{
 wxFileConfig config(wxEmptyString,wxEmptyString,wxGetCwd()+wxT("/G65DDCCohETCDemo_Client.ini"));
 long count,delay,i;
 wxString s;
 wxString host_name;
 long port;

    config.SetPath(wxT("/GLOBAL"));
    count=config.ReadLong(wxT("Count"),0);

    if(count<=0)
    {
        wxMessageBox(wxT("Failed to read configuration.\nNo remote host is specified."),APP_TITLE,wxICON_STOP);
        return false;
    }

    remote_hosts.clear();

    for(i=0;i<count;i++)
    {
        s.sprintf(wxT("/%ld"),i);
        config.SetPath(s);
        host_name=config.Read(wxT("HostName"));
        port=config.ReadLong(wxT("Port"),0);
        delay=config.ReadLong(wxT("Delay"),0);

        if(delay<=0)
        {
            wxMessageBox(wxT("Failed to read configuration.\nThe delay is not specified."),APP_TITLE,wxICON_STOP);
            return false;
        }

        if(host_name==wxEmptyString)
        {
            wxMessageBox(wxT("Failed to read configuration.\nHost name is not specified."),APP_TITLE,wxICON_STOP);
            return false;
        }

        if(port<1 || port>0xffff)
        {
            wxMessageBox(wxT("Failed to read configuration.\nPort number is not specified or it is invalid."),APP_TITLE,wxICON_STOP);
            return false;
        }

        remote_hosts.push_back(new remote_host(i,delay,host_name.mb_str().data(),(uint16_t)port));
    }

    if(remote_hosts.empty())
    {
        wxMessageBox(wxT("Failed to read configuration.\nNo remote host is specified."),APP_TITLE,wxICON_STOP);
        return false;
    }

    return true;
}

bool app::OnInit(void)
{
 std::vector<remote_host*> remote_hosts;
 std::vector<rx_client*> clients;
 wxString s;
 struct in_addr addr;
 std::string host_name;
 hostent *host_ent;

    if(!read_config(remote_hosts))
    {
        return false;
    }

    for(auto remote_host:remote_hosts)
    {
        if(inet_pton(AF_INET,remote_host->m_host_name.c_str(),&addr)!=1)
        {
            host_ent=gethostbyname2(remote_host->m_host_name.c_str(),AF_INET);

            if(host_ent==NULL)
            {
                s=wxT("Failed to translate remote host ");
                s+=remote_host->m_host_name.c_str();
                s+=wxT(" name to its address.");
                wxMessageBox(s,APP_TITLE,wxICON_STOP);
                return false;
            }

            addr=*(struct in_addr*)host_ent->h_addr_list[0];
            host_name=remote_host->m_host_name;
        }
        else
        {
            host_ent=gethostbyaddr((char*)&addr,sizeof(addr),AF_INET);

            if(host_ent && host_ent->h_name)
            {
                host_name=host_ent->h_name;
            }
            else
            {
                host_name="";
            }
        }

        clients.push_back(new rx_client(remote_host->m_index,remote_hosts.size(),remote_host->m_delay,host_name,addr,remote_host->m_port));

        delete remote_host;
    }

    remote_hosts.clear();

    auto window=new main_window(clients);

    window->Show(true);

    return true;
}

IMPLEMENT_APP(app)
