/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_RX_SERVER_H__
#define __U_RX_SERVER_H__

#include <stdint.h>
#include "../g65ddcapi.h"
#include "../u_packet.h"
#include <thread>
#include <mutex>

void dbg_print(const char *format,...);

#define _DEBUG

#ifdef _DEBUG
#define DEBUG_TO_STR2(x) #x
#define DEBUG_TO_STR(x) DEBUG_TO_STR2(x)
//#define DEBUG(__fmt__,...) dbg_print(__fmt__ " [" __FUNCTION__ ":" DEBUG_TO_STR(__LINE__) "]\n",##__VA_ARGS__);
#define DEBUG(__fmt__,...) dbg_print(__fmt__ "\n",##__VA_ARGS__);
#else
#define DEBUG(__fmt__,...)
#endif //_DEBUG

class rx_server:public ICohG65DDCDeviceSetCallback
{
public:
    virtual void CohG65DDC_DDC1StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceCount,const void **Buffers,uint32_t NumberOfSamples,uint32_t BitsPerSample);
    virtual void CohG65DDC_ADCSnapshotCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel);
    virtual void CohG65DDC_DDC2StreamCallback(ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples);
    virtual void CohG65DDC_DDC2PreprocessedStreamCallback(ICohG65DDCDeviceSet *Device,uint32_t DeviceIndex,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS);
    virtual void CohG65DDC_AudioStreamCallback(ICohG65DDCDeviceSet *Device,uint32_t DeviceIndex,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples);

    const char* get_serial_number(void) const
    {
        return m_device_info.SerialNumber;
    }

    bool start(uint16_t port);
    void stop(void);
    rx_server(ICohG65DDCDeviceSet *device_set);
    virtual ~rx_server(void);
private:
    ICohG65DDCDeviceSet         *m_device_set;
    G65DDC_DEVICE_INFO          m_device_info;
    int                         m_listen_socket;
    std::thread                 *m_thread;
    bool                        m_stop_request;
    int                         m_client_socket;
    std::recursive_mutex        m_send_cs;
    packet_seq_t                m_packet_seq;
    char                        m_client_address_str[32];
    union tcp_packet            *m_packet_ddc1_samples;
    uint64_t                    m_ddc1_sample_set_counter;
    uint32_t                    m_ddc1_sample_sets_to_send;

    bool client_sockt_begin(void);
    void clinet_socket_end(void);

    void working_thread_proc(void);

    void process_packet(union tcp_packet *packet);
    uint32_t process_packet_set_extended_configuration_params(union tcp_packet *packet);
    uint32_t process_packet_set_power(union tcp_packet *packet);
    uint32_t process_packet_start_ddc1(union tcp_packet *packet);
    uint32_t process_packet_stop_ddc1(void);
    uint32_t process_packet_trigger(void);
    uint32_t process_packet_trigger_test(void);
    uint32_t process_packet_set_ddc1_frequency(union tcp_packet *packet);
    uint32_t process_packet_set_ddc1_phase_shift(union tcp_packet *packet);
    uint32_t process_packet_set_atten(union tcp_packet *packet);
    uint32_t process_packet_set_preamp(union tcp_packet *packet);
    uint32_t process_packet_set_1pps(union tcp_packet *packet);

    void close_client_socket(void);

    bool send(union tcp_packet &packet);
    void send_device_info(void);
    void send_state(void);
};

#endif //__U_RX_SERVER_H__
